@@ -1,8 +1,9 @@
 import os
 import sys
+from collections import defaultdict
 from enum import Enum
 from functools import lru_cache
-from typing import Type, List, cast, Optional
+from typing import Type, List, cast, Optional, Dict
 
 import spacy
 from pydantic import BaseModel, Field
@@ -37,11 +38,43 @@ class SciSpacyLinker(str, Enum):
     mesh = 'mesh'
 
 
+class SpacyLabel(str, Enum):
+    # fr,de,pt,it
+    PER = "PER"
+    MISC = "MISC"
+    # en,zh,nl
+    LOC = "LOC"
+    ORG = "ORG"
+    PERSON = "PERSON"
+    CARDINAL = "CARDINAL"
+    DATE = "DATE"
+    EVENT = "EVENT"
+    FAC = "FAC"
+    GPE = "GPE"
+    LANGUAGE = "LANGUAGE"
+    LAW = "LAW"
+    MONEY = "MONEY"
+    NORP = "NORP"
+    ORDINAL = "ORDINAL"
+    PERCENT = "PERCENT"
+    PRODUCT = "PRODUCT"
+    QUANTITY = "QUANTITY"
+    TIME = "TIME"
+    WORK_OF_ART = "WORK_OF_ART"
+    # scispacy
+    ENTITY = "ENTITY"
+
+
 class SpacyNERParameters(AnnotatorParameters):
     model_type: SpacyModelType = Field(SpacyModelType.base,
                                        description="""Type of NER spacy model to use for NER extraction, can be:
                        <li>`base` for basic entities like location/person/organization provided by [spacy](https://spacy.io/models) or
                        <li>`sci` for biomedical entities provided by [scispacy](https://allenai.github.io/scispacy)""")
+    mapping: Dict[str, List[SpacyLabel]] = Field(
+        None,
+        description="Map a label to a list of spacy NER labels",
+        extra="key:label",
+    )
     model_size: SpacyModelSize = Field(SpacyModelSize.sm,
                                        description="""Size of NER spacy model to use for NER extraction, can be:
                        <li>`sm` for small,
@@ -66,7 +99,7 @@ class SpacyNERParameters(AnnotatorParameters):
                                        extra="advanced")
 
 
-SUPPORTED_LANGUAGES = "en,fr,de,nl,es,pt,it,zh,ar,ru"
+SUPPORTED_LANGUAGES = "en,fr,de,nl,es,pt,it,zh,ru,ja"
 
 
 class SpacyNERAnnotator(AnnotatorBase):
@@ -79,7 +112,13 @@ class SpacyNERAnnotator(AnnotatorBase):
         params: SpacyNERParameters = \
             cast(SpacyNERParameters, parameters)
         supported_languages = comma_separated_to_list(SUPPORTED_LANGUAGES)
-
+        if params.mapping is not None:
+            spacy2labels = defaultdict(list)
+            for k, label_lst in params.mapping.items():
+                for label in label_lst:
+                    spacy2labels[label].append(k)
+        else:
+            spacy2labels = None
         for document in documents:
             lang = document_language(document, None)
             if lang is None or lang not in supported_languages:
@@ -95,29 +134,37 @@ class SpacyNERAnnotator(AnnotatorBase):
             asents = nlp.pipe(sents)
             for sent, asent in zip(document.sentences, asents):
                 for ent in asent.ents:
-                    start = sent.start + ent.start_char
-                    end = sent.start + ent.end_char
-                    terms = []
-                    if linker is not None:
-                        for umls_ent in ent._.kb_ents:
-                            score = umls_ent[1]
-                            if score >= params.mention_threshold:
-                                kbent: Entity = linker.kb.cui_to_entity[umls_ent[0]]
-                                types = types_as_set.intersection(kbent.types)
-                                if types or len(types_as_set) == 0:
-                                    props = {'types': kbent.types if len(types_as_set) == 0 else list(types),
-                                             'aliases': kbent.aliases}
-                                    term = Term(identifier=umls_ent[0], lexicon=params.scispacy_linker.value,
-                                                preferredForm=kbent.canonical_name, score=score, properties=props)
-                                    if len(terms) < params.mention_max:
-                                        terms.append(term)
-                                    else:
-                                        break
-                    document.annotations.append(Annotation(start=start, end=end,
-                                                           labelName=ent.label_.lower(),
-                                                           label=ent.label_,
-                                                           text=document.text[start:end],
-                                                           terms=terms if linker is not None else None))
+                    if spacy2labels is None or ent.label_ in spacy2labels:
+                        start = sent.start + ent.start_char
+                        end = sent.start + ent.end_char
+                        terms = []
+                        if linker is not None:
+                            for umls_ent in ent._.kb_ents:
+                                score = umls_ent[1]
+                                if score >= params.mention_threshold:
+                                    kbent: Entity = linker.kb.cui_to_entity[umls_ent[0]]
+                                    types = types_as_set.intersection(kbent.types)
+                                    if types or len(types_as_set) == 0:
+                                        props = {'types': kbent.types if len(types_as_set) == 0 else list(types),
+                                                 'aliases': kbent.aliases}
+                                        term = Term(identifier=umls_ent[0], lexicon=params.scispacy_linker.value,
+                                                    preferredForm=kbent.canonical_name, score=score, properties=props)
+                                        if len(terms) < params.mention_max:
+                                            terms.append(term)
+                                        else:
+                                            break
+                        if spacy2labels is None:  # same as before
+                            document.annotations.append(Annotation(start=start, end=end,
+                                                                   labelName=ent.label_.lower(),
+                                                                   label=ent.label_,
+                                                                   text=document.text[start:end],
+                                                                   terms=terms if linker is not None else None))
+                        else:
+                            for lname in spacy2labels[ent.label_]:
+                                document.annotations.append(Annotation(start=start, end=end,
+                                                                       labelName=lname,
+                                                                       text=document.text[start:end],
+                                                                       terms=terms if linker is not None else None))
         return documents
 
     @classmethod
